/* The ratio of the complete elliptic integrals of the first kind K(k) and K(k'). */

#include <math.h>
#include <complex.h>
#include <float.h>

double complex _cEllipticKRatio(double complex k);
double complex _agm(double complex a, double complex b);

/** The arithmetic-geometric mean
 _agm(a, b) returns the arithmetic-geometric mean agm(a, b) of two numbers a and b.
 a_{n+1} &=& \frac{1}{2}(a_n + b_n)\\
 b_{n+1} &=&  \sqrt{a_n b_n}
 */
double complex
_agm(a, b)
    double complex a;
    double complex b;
{
    double         e;
    double complex m;
    if ((cabs(a) == 0) || (cabs(b) == 0) || (a == -b)) return 0;
    e = 2 * DBL_EPSILON * fmax(cabs(a), cabs(b));
    do {
        m = csqrt(a * b);
        a = (a + b) * 0.5;
        b = ((cabs(a + m) == cabs(a - m)) && (cimag(m / a) > 0)) || (cabs(a + m) < cabs(a - m)) ? -m : m;
    } while (cabs(a - b) > e);
    return ((a + b) * 0.5 );
}

/** The ratio of the complete elliptic integrals of the first kind K(k) and K(k').
 *  The argument k is the elliptic modulus. 
 *  It can be calculated by the following relation with the ratio of the arithmetic-geometric means.
\frac{K(k)}{K(k')}  & = & \displaystyle\frac{\mathrm{agm}(1, k)}{\mathrm{agm}(1, k')}\\
k' &=& \sqrt{1 - k^2}
*/
double complex
_cEllipticKRatio(k) 
    double complex k;
{
    return _agm(1, k) / _agm(1, csqrt(1 - k * k));
}

#ifdef DEBUG
#include <stdio.h>
#include <stdlib.h>

char * help = "Name\n\
    cEllipticKRatio -- returns the ratio of the complete elliptic integrals of the first kind K(k) and K(k').\n\
    The argument k is the elliptic modulus.\n\
    Rev. 1.2 (Jan. 9, 2023) (c) 2023, Takayuki HOSODA, Finetune co., ltd.\n\
Usage\n\
    cEllipticKRatio k.re k.im\n\
Example\n\
    cEllipticKRatio(0.3) = 0.611943427652876\n\
    cEllipticKRatio(0.5) = 0.7817009613480558\n\
";
int
main(int argc, char *argv[]) {
    double complex k;
    double complex c;
    double d;
    int    n = 800;
    int    r, j;
    if (argc == 3) {
        k = (atof(argv[1]) + I * atof(argv[2]));
        c = _cEllipticKRatio(k);
        printf("cEllipticKRatio(%+g %+gi) = %+.16g %+.16gi\n", creal(k), cimag(k), creal(c), cimag(c));
    } else if (argc == 2) {
        d = atof(argv[1]);
        printf("#Computed by \"cEllipcRatio Rev.1.2 (Jan. 9, 2023) (c) 2022, Takayuki HOSODA\n");
        printf("#z.re\tz.im\tmagnitude\tphase angle\n");
        for (r = 0 ; r <= n; r++) {
            for (j = 0 ; j <= n; j++) {
                k = (-d * r + (n - r) * d) / n + I * ((-d * j + (n -j) * d) / n);
                c = _cEllipticKRatio(k);
                printf("%.16g\t%.16g\t%.16g\t%.16g\n",creal(k), cimag(k), cabs(c), carg(c));
            }
            printf("\n");
        }
    } else {
        printf("%s\n", help);
    }
    return(0);
}
#endif
/* Referance value
cEllipticKRatio(0.3) ~= 0.611943427652876082187005088716421695349032591821961520932345529...
cEllipticKRatio(0.5) ~= 0.781700961348055753475244064338928776837763338172705084719132764...
cEllipticKRatio(1+i) ~= 0.707206971653503254060531189935941741754632244414382494651196274...
                      + 0.636030459365383367231569857133038707122760532720386473394720765... i
cEllipticKRatio(-1+i)~= 0.340890988209541775363037328345745055722467890188130582646002126...
                      + 0.565569995175872280530790674019772278083086911404224088685296327... i
*/
